local WYM = _G.WymBuffs
local L = WYM.L
local LCB = LibStub("LibCandyBar-3.0")

local sparks = {}

function WYM:UpdateBars(frame, buffs)
    local i = 1
    local rowOffsetX = 0
    local rowOffsetY = 0    
    local previous = nil
    local anchorPoint, relativePoint, barOffsetX, barOffsetY = self:GetBarAnchorPointsAndOffsets(frame)

    for _, buff in ipairs(buffs) do
		local bar = self:GetBar(frame,buff)
		if not previous then
		    bar:SetPoint(anchorPoint,frame,anchorPoint,rowOffsetX, rowOffsetY)
		else
		    bar:SetPoint(anchorPoint,previous,relativePoint,barOffsetX,barOffsetY)
		end	    	    
		previous = bar

        local rowColumnIndex = i % frame.settings.rowsColumns

        if rowColumnIndex == 0 then
            previous = nil
            local x, y = self:GetNewBarRowColumnOffsets(frame)
            rowOffsetX = rowOffsetX + x
            rowOffsetY = rowOffsetY + y  
        end
	
		if buff.duration == 0 and buff.expirationTime == 0 then
		    bar.running = true
		    self:ResetNoDurationBar(bar)
		    bar:Show()		
		else
		    bar:Start()
		end
		
		bar.exp = buff.expirationTime	    
		frame.bars[i] = bar
		i = i + 1
    end
end

function WYM:ResetBars(frame)
    for i, bar in ipairs(frame.bars) do
		self:stopBlink(bar)
	
        --[[
           Sometimes a bar can stop because the timer runs out before we get the message
           about the auras changing, so we want to make sure we only stop the bar if it's
           actually running. If we stop a stopped bar the pool of reusable bars in LibCandyBar
           will get messed up since the same bar will be added to it twice. That leads to all
           sorts of wierd problems for this addon. It only took me about 8 hours of pulling
           my hair to figure this out.
        ]]--
        if bar.running then
            bar:Stop()
        end
    end

    -- Getting some strange behavior when trying to hide sparks in the current bars. Doing it 
    -- this way seems to get rid of that behavior    
    for i, spark in ipairs(sparks) do
        spark:Hide()
    end
end

function WYM:ResetNoDurationBar(bar)
    if not bar.running then return end
    if bar.candyBarIconFrame:GetTexture() then
	    bar.candyBarBar:SetPoint("TOPLEFT", bar.candyBarIconFrame, "TOPRIGHT")
	    bar.candyBarBar:SetPoint("BOTTOMLEFT", bar.candyBarIconFrame, "BOTTOMRIGHT")
	    bar.candyBarIconFrame:SetWidth(bar.height)
	    bar.candyBarIconFrame:Show()
    else
	    bar.candyBarBar:SetPoint("TOPLEFT", self)
	    bar.candyBarBar:SetPoint("BOTTOMLEFT", self)
	    bar.candyBarIconFrame:Hide()
    end

    bar.candyBarBar:SetMinMaxValues(0,1)
    bar.candyBarBar:SetValue(1)
    bar.candyBarDuration:Hide()
end

function WYM:GetBuffBarFrameSize(frame)
    local width = frame.settings.barWidth
    local height = frame.settings.barHeight
    local barSpacing = frame.settings.barSpacing
    local growthDirection = frame.settings.growthDirection
    
    if growthDirection == "VERTICAL" then
        height = (# frame.bars * height) + (# frame.bars - 1 * barSpacing)
        return  width, height
    else
        width = (# frame.bars * width) + (# frame.bars - 1 * barSpacing)
        return width, height    
    end
end

function WYM:GetBarAnchorPointsAndOffsets(frame)
    local barSpacing = frame.settings.barSpacing
    local growthDirection = frame.settings.growthDirection
    local frameAnchorPoint = frame.settings.positionAnchorPoint
        
    -- Vertical direction and an anchor point at the bottom means we'll grow the frame upwards. With another anchor point
    -- we'll grow it downwards. The same principle applies to horizontal direction and right sided anchor point
    if growthDirection == "VERTICAL" then    
	if frameAnchorPoint == "BOTTOMLEFT" or frameAnchorPoint == "BOTTOM" or frameAnchorPoint == "BOTTOMRIGHT" then
	    return "BOTTOMLEFT", "TOPLEFT", 0, barSpacing
	else
	    return "TOPLEFT", "BOTTOMLEFT", 0, barSpacing * - 1
	end	
    else
	if frameAnchorPoint == "TOPRIGHT" or frameAnchorPoint == "RIGHT" or frameAnchorPoint == "BOTTOMRIGHT" then
	    return "TOPRIGHT", "TOPLEFT", barSpacing * -1, 0
	else
	    return "TOPLEFT", "TOPRIGHT", barSpacing, 0
	end
    end
end

function WYM:GetNewBarRowColumnOffsets(frame)
    local barSpacingX = frame.settings.barWidth + frame.settings.barSpacing
    local barSpacingY = frame.settings.barHeight + frame.settings.barSpacing
    local growthDirection = frame.settings.growthDirection
    local frameAnchorPoint = frame.settings.positionAnchorPoint    

    if growthDirection == "VERTICAL" then
        if frameAnchorPoint == "TOPLEFT" or frameAnchorPoint == "LEFT" or frameAnchorPoint == "BOTTOMLEFT" then
            return barSpacingX,0
        else
            return barSpacingX * -1,0
        end
    else
        if frameAnchorPoint == "TOPLEFT" or frameAnchorPoint == "TOP" or frameAnchorPoint == "TOPRIGHT" then
            return 0, barSpacingY * -1,0
        else
            return 0, barSpacingY
        end
    end
end

function WYM:GetBar(frame,buff)
    local bar = LCB:New(self:GetBarTexture(frame.settings.barTexture),frame.settings.barWidth,frame.settings.barHeight)    
    
    bar:Set("Buff", buff)
    
    bar.candyBarBackground:SetVertexColor(frame.settings.barBackgroundColorR, frame.settings.barBackgroundColorG, frame.settings.barBackgroundColorB, frame.settings.barBackgroundColorA)    
    bar:SetColor(self:GetBuffColor(frame,buff))        
    bar.candyBarLabel:SetFont(self:GetFont(frame.settings.barLabelFont), frame.settings.barLabelFontSize, frame.settings.barLabelFontOutline)
    bar.candyBarLabel:SetTextColor(frame.settings.barLabelTextColorR, frame.settings.barLabelTextColorG, frame.settings.barLabelTextColorB, frame.settings.barLabelTextColorA)
    bar.candyBarLabel:SetJustifyH(frame.settings.barLabelAlignment)
    if buff.count <= 1 then
	   bar:SetLabel(buff.name)
    else
	   bar:SetLabel(buff.name .. " (" .. buff.count .. ")")
    end
    
    if frame.settings.barShowIcon then
        bar:SetIcon(buff.icon)
    else
        bar:SetIcon(nil)
    end

    bar:SetDuration(buff.duration)
    bar.candyBarDuration:SetFont(self:GetFont(frame.settings.barLabelFont), frame.settings.barLabelFontSize, frame.settings.barLabelFontOutline)
    bar.candyBarDuration:SetTextColor(frame.settings.barLabelTextColorR, frame.settings.barLabelTextColorG, frame.settings.barLabelTextColorB, frame.settings.barLabelTextColorA)
    bar:SetTimeVisibility(frame.settings.barShowTime)
    
    buff.widget = bar
    self:EnableTooltip(frame, buff)
    self:EnableWarnings(frame, buff)

    bar:AddUpdateFunction(function(bar)
        if frame.settings.barShowSpark then			    
		    local spark = bar.spark
			if not spark then
    		    spark = bar.candyBarBar:CreateTexture(nil,"OVERLAY")
			    tinsert(sparks,spark) -- Getting some strange behaviour with sparks sticking around seemingly randomly. We keep 
			                          -- track of all sparks created in this way to hopefully stop this from happening
			    spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
			    spark:SetWidth(20)                                        
			    spark:SetBlendMode("ADD")
			    spark:SetHeight(bar.height * 2.5)
			    spark:SetPoint("CENTER",bar,"LEFT",bar.width,0)
			    bar.spark = spark
			end
			spark:Show()
    
			local statusBar = bar.candyBarBar
			local _, max = statusBar:GetMinMaxValues()
			local position = (statusBar:GetValue() / max * statusBar:GetWidth());
			if not frame.settings.barShowIcon then
			    position = position - frame.settings.barHeight -- Icon is square so bar height = icon width
			end
			spark:ClearAllPoints()
			spark:SetPoint("CENTER",bar,"LEFT",position + spark:GetWidth(),0)
    
    		self:HandleWarnings(frame, buff)
	    end
    end )
    
    return bar
end
