local WYM = _G.WymBuffs
local L = WYM.L

local buttonCache = {}

local function FormatTime(format, time)
    if format == "DEFAULT" then
        if time <= 0 then
            return "0.0"
        elseif time >= 3600 then
            local hour = floor(time / 3600)
            local minute = floor((time - (hour * 3600)) / 60)
            return "%dh%02dm", hour, minute
        elseif time >= 60 then
            local minute = ceil(time / 60)
            return "%dm", minute
        elseif time < 10 then
            return "%1.1fs", time
        else
            return "%.0fs", floor(time + .5)
        end
    elseif format == "MM:SS" then
        local hour = floor(time / 3600)
        local minute = floor((time - (hour * 3600)) / 60)
        local second = floor(time % 60)
    
        if hour >= 1 then
            return "%d:%02d:%02d",hour, minute, second
        else
            return "%02d:%02d",minute, second        
        end
    else
        return "??:??"
    end
end

local function OnUpdate(button)
    button.time:SetFormattedText(FormatTime(button.frame.settings.iconTimerFormat,button.buff.remaining()))
    WYM:HandleWarnings(button.frame, button.buff)
end

function WYM:UpdateIcons(frame, buffs)
    local i = 1
    local previous = nil
    local rowOffsetX = 0
    local rowOffsetY = 0
    local anchorPoint, relativePoint, iconOffsetX, iconOffsetY = self:GetIconAnchorPointsAndOffsets(frame)

    for _, buff in ipairs(buffs) do
        local button = self:GetButton(frame,buff)

        
        if not previous then
            button:SetPoint(anchorPoint,frame,anchorPoint,rowOffsetX, rowOffsetY)
        else
            button:SetPoint(anchorPoint,previous,relativePoint,iconOffsetX,iconOffsetY)
        end	    	    
        previous = button
        
        frame.icons[i] = button
        
        local rowColumnIndex = i % frame.settings.rowsColumns

        if rowColumnIndex == 0 then
            previous = nil
            local x, y = self:GetNewIconRowColumnOffsets(frame)
            rowOffsetX = rowOffsetX + x
            rowOffsetY = rowOffsetY + y
        end
        
        i = i + 1
        
        button:Show()
    end
end

function WYM:ResetIcons(frame)
    for i, button in ipairs(frame.icons) do
	self:stopBlink(button)
        button:Hide()
        button:SetScript("OnUpdate", nil)
        button.buff = nil
        button.frame = nil
        self:RemoveButtonFromButtonFacadeGroup(frame,button)
        tinsert(buttonCache,button)
    end
end

function WYM:GetBuffIconFrameSize(frame)
    local width = 32 * frame.settings.iconScale
    local height = 32 * frame.settings.iconScale
    local iconSpacing = frame.settings.iconSpacing
    local growthDirection = frame.settings.growthDirection
    
    if growthDirection == "VERTICAL" then
        height = (# frame.icons * height) + (# frame.icons - 1 * iconSpacing)
        return  width, height
    else
        width = (# frame.icons * width) + (# frame.icons - 1 * iconSpacing)
        return width, height    
    end
end

function WYM:GetIconAnchorPointsAndOffsets(frame)
    local iconSpacing = frame.settings.iconSpacing
    local growthDirection = frame.settings.growthDirection
    local frameAnchorPoint = frame.settings.positionAnchorPoint

    -- Vertical direction and an anchor point at the bottom means we'll grow the frame upwards. With another anchor point
    -- we'll grow it downwards. The same principle applies to horizontal direction and right sided anchor point
    if growthDirection == "VERTICAL" then
        if frameAnchorPoint == "BOTTOMLEFT" or frameAnchorPoint == "BOTTOM" or frameAnchorPoint == "BOTTOMRIGHT" then
            return "BOTTOMLEFT", "TOPLEFT", 0, iconSpacing
        else
            return "TOPLEFT", "BOTTOMLEFT", 0, iconSpacing * - 1
        end
    else
	if frameAnchorPoint == "TOPRIGHT" or frameAnchorPoint == "RIGHT" or frameAnchorPoint == "BOTTOMRIGHT" then
	    return "TOPRIGHT", "TOPLEFT", iconSpacing * -1, 0
	else
	    return "TOPLEFT", "TOPRIGHT", iconSpacing, 0
	end
    end
end

function WYM:GetNewIconRowColumnOffsets(frame)
    local iconSize = 32 * frame.settings.iconScale
    local iconSpacing = frame.settings.iconSpacing + iconSize
    local growthDirection = frame.settings.growthDirection
    local frameAnchorPoint = frame.settings.positionAnchorPoint    
    
    if growthDirection == "VERTICAL" then
        if frameAnchorPoint == "TOPLEFT" or frameAnchorPoint == "LEFT" or frameAnchorPoint == "BOTTOMLEFT" then
            return iconSpacing,0
        else
            return iconSpacing * -1,0
        end
    else
        if frameAnchorPoint == "TOPLEFT" or frameAnchorPoint == "TOP" or frameAnchorPoint == "TOPRIGHT" then
            return 0, iconSpacing * -1,0
        else
            return 0, iconSpacing
        end
    end
end

function WYM:GetIconTextAnchorPointsAndOffsets(position)
    if position == "BELOW" then
        return "TOP", "BOTTOM", 0, -1
    elseif position == "ABOVE" then
        return "BOTTOM", "TOP", 0, 1
    elseif position == "LEFT" then
        return "RIGHT", "LEFT", -1, 0
    elseif position == "RIGHT" then
        return "LEFT", "RIGHT", 1, 0
    end
end

function WYM:GetButton(frame, buff)
    local button = tremove(buttonCache)    
    
    if not button then
        button = CreateFrame("Button", nil, frame)
        button.icon = button:CreateTexture(nil, "ARTWORK")
        button.border = button:CreateTexture(nil, "BACKGROUND")
        button.name = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
        button.time = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
        button.stack = button:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmallOutline")
        button.stack:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -1, 1)        
    else
        button:ClearAllPoints()
    end
    
    buff.widget = button
    self:EnableTooltip(frame, buff)
    self:EnableWarnings(frame, buff)
    
    button.buff = buff
    button.frame = frame
    
    button:SetWidth(32 * frame.settings.iconScale)
    button:SetHeight(32 * frame.settings.iconScale)
    button:SetAlpha(1)
    
    button.icon:SetAllPoints(button)
    button.icon:SetTexture(buff.icon)
    
    button.border:SetPoint("CENTER", button, "CENTER", -0.25, 0)
    button.border:SetSize(frame.settings.iconBorderSize * frame.settings.iconScale,frame.settings.iconBorderSize * frame.settings.iconScale)
    button.border:SetVertexColor(self:GetBuffColor(frame,buff))
    button.border:SetTexture("Interface\\AddOns\\WymBuffs\\media\\iconBorder")

    if frame.settings.showIconBorder then
        button.border:Show()
    else
        button.border:Hide()
    end

    if frame.settings.iconLabelPosition == "HIDE" then
        button.name:Hide()
    else    
        button.name:ClearAllPoints()
        button.name:SetFont(self:GetFont(frame.settings.iconLabelFont),frame.settings.iconLabelFontSize,frame.settings.iconLabelFontOutline)
        button.name:SetTextColor(frame.settings.iconLabelTextColorR, frame.settings.iconLabelTextColorG, frame.settings.iconLabelTextColorB, frame.settings.iconLabelTextColorA)
        local anchor, relative, offsetX, offsetY = self:GetIconTextAnchorPointsAndOffsets(frame.settings.iconLabelPosition)
        button.name:SetPoint(anchor,button, relative, offsetX, offsetY)
        button.name:SetText(buff.name)
        button.name:Show()                
    end

    if buff.remaining() >= 0 then
        if frame.settings.iconTimerPosition == "HIDE" then
            button.time:Hide()
        else    
            button.time:ClearAllPoints()
            button.time:SetFont(self:GetFont(frame.settings.iconTimerFont),frame.settings.iconTimerFontSize,frame.settings.iconTimerFontOutline)
            button.time:SetTextColor(frame.settings.iconTimerTextColorR, frame.settings.iconTimerTextColorG, frame.settings.iconTimerTextColorB, frame.settings.iconTimerTextColorA)
            local anchor, relative, offsetX, offsetY = self:GetIconTextAnchorPointsAndOffsets(frame.settings.iconTimerPosition)
            button.time:SetPoint(anchor,button,relative, offsetX, offsetY)            
            button:SetScript("OnUpdate", OnUpdate)
            button.time:Show()                
        end
    else
        button.time:Hide()
        button.time:SetText("")
    end
                                
    if buff.count > 1 then
        button.stack:SetText(buff.count)
        button.stack:SetFont(self:GetFont(frame.settings.iconLabelFont),frame.settings.iconLabelFontSize,frame.settings.iconLabelFontOutline)
        button.stack:SetTextColor(frame.settings.iconLabelTextColorR, frame.settings.iconLabelTextColorG, frame.settings.iconLabelTextColorB, frame.settings.iconLabelTextColorA)
        button.stack:Show()
    else
        button.stack:Hide()
    end
    
    self:AddButtonToButtonFacadeGroup(frame,button)
    
    return button
end

