WymBuffs = LibStub("AceAddon-3.0"):NewAddon("WymBuffs", "AceConsole-3.0", "AceEvent-3.0")
WymBuffs.L = LibStub("AceLocale-3.0"):GetLocale("WymBuffs")
local WYM = WymBuffs

local auraCache = {}

function WYM:OnEnable()
    self:RegisterEvent("UNIT_AURA")
    self:RegisterEvent("PLAYER_TARGET_CHANGED")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("UNIT_INVENTORY_CHANGED")
    
    self:UpdateBuffFrames()    
end

function WYM:OnDisable()
    self:UnregisterAllEvents()
    
    self:UpdateBuffFrames()
end

function WYM:UNIT_AURA(_,unit)
    self:UpdateBuffFrames(unit)    
end

function WYM:PLAYER_TARGET_CHANGED()
    self:UpdateBuffFrames("target")
end

function WYM:PLAYER_REGEN_DISABLED()
    self:UpdateBuffFrames()
end

function WYM:PLAYER_REGEN_ENABLED()
    self:UpdateBuffFrames()
end

function WYM:UNIT_INVENTORY_CHANGED(_,unit)
    if unit == "player" then
        self:UpdateBuffFrames("weapon")
    end
end

function WYM:GetUnitAuras(frame)    
    local i = 1
    local auras = frame.auras
    local unit = frame.settings.unit
    local type = frame.settings.type
    local aura = self:GetUnitAura(unit,i, type)
    
    while aura do
        tinsert(auras,aura)
        i = i + 1
        aura = self:GetUnitAura(unit,i, type)
    end
    
    return auras
end

function WYM:GetUnitAura(unit, index, filter)
    
    local name, rank, icon, count, dispelType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura(unit, index, filter)

    if name then
        local aura = tremove(auraCache)
        if not aura then
            aura = {}
        end
        
        aura.name = name
        aura.rank = rank
        aura.icon = icon
        aura.count = count
        aura.dispelType = dispelType
        aura.duration = duration
        aura.expirationTime = expirationTime        
        aura.remaining = function()
                            if expirationTime > 0 then
                                return expirationTime - GetTime()
                            else
                                return -1
                            end
                         end
        aura.unitCaster = unitCaster
        aura.isStealable = isStealable
        aura.shouldConsolidate = shouldConsolidate
        aura.spellId = spellId
        aura.isTest = false
        -- unit, index and filter are needed for the tooltip
        aura.unit = unit
        aura.index = index
        aura.filter = filter
        -- Set and used in various places
        aura.widget = nil
        aura.newBuffWarning = nil
        aura.warningSoundTime = nil
        aura.warningBlinkTime = nil        
        
        if WYM.db.profile.missingSpellId[aura.name] == -1 then
            WYM.db.profile.missingSpellId[aura.name] = aura.spellId
        end

        return aura
    else
        return nil
    end
end

function WYM:RecycleAuras(frame)
    for _, aura in ipairs(frame.auras) do
        tinsert(auraCache, aura)
    end
end

function WYM:GetMissingAuras(frame)
    local auras = self:GetUnitAuras(frame)
    local missingAuras = frame.buffs  
    
    for _,buffSettings in pairs(frame.settings.buffWatchList) do
        local missing = true
        
        for _,aura in ipairs(auras) do
            if aura.name == buffSettings.name then
                missing = false
                break
            end
        end
        
        if missing then

            local icon = GetSpellTexture(buffSettings.name)
            
            if not frame.settings.buffWatchList[buffSettings.name].showAsMissingOnlyIfSpellExists and not icon then
                -- If we can't find a texture with GetSpellTexture(), it means we don't have a spell in our spellbook
                -- with the same name as the buff. If that's the case, and showAsMissingOnlyIfSpellExists is not checked,
                -- we'll try looking in the profile spellId cache for the spell id.
                
                local spellId = WYM.db.profile.missingSpellId[buffSettings.name]
                if not spellId then
                    -- If we don't have the spell id in the cache, we put a dummy value there to indicate we want to fill it
                    -- in the next time we see this spell
                    spellId = -1
                    WYM.db.profile.missingSpellId[buffSettings.name] = spellId
                end
                
                -- Get the icon by providing the spell id instead of the buff name. If we can't find it (spell id is -1, see above)
                -- then we'll just use a dummy icon. Once the addon has seen the buff once, the dummy icon will be replaced
                _, _, icon = GetSpellInfo(spellId)    
                if not icon then
                    icon = "Interface\\Icons\\Temp.blp"
                end
            end

            if icon then
                -- Make a dummy aura and put some placeholder values in it. They probably aren't correct but we don't care in this case
                local aura = {}
                
                aura.name = buffSettings.name
                aura.rank = 1
                aura.icon = icon
                aura.count = 1
                aura.dispelType = "warning"
                aura.duration = 0
                aura.expirationTime = 0
                aura.remaining = function() return -1 end
                aura.unitCaster = "player"
                aura.isStealable = false
                aura.shouldConsolidate = false
                aura.spellId = 1
                aura.isMissing = true
                tinsert(missingAuras,aura)
            end
        end        
    end
    
    return missingAuras
end

function WYM:GetDummyAuras()
    local dummyAuras = {}
    local aura = {}
    
    aura.name = "Magic (Stack of Three)"
    aura.rank = 1
    aura.icon = "Interface\\Icons\\Temp.blp"
    aura.count = 3
    aura.dispelType = "Magic"
    aura.duration = 30
    aura.expirationTime = GetTime() + 30
    aura.remaining = function() return aura.expirationTime - GetTime() end
    aura.unitCaster = "player"
    aura.isStealable = false
    aura.shouldConsolidate = false
    aura.spellId = 1
    aura.isTest = true
    tinsert(dummyAuras,aura)

    aura = {}
    aura.name = "Disease"
    aura.rank = 1
    aura.icon = "Interface\\Icons\\Temp.blp"
    aura.count = 1
    aura.dispelType = "Disease"
    aura.duration = 10
    aura.expirationTime = GetTime() + 10
    aura.remaining = function() return aura.expirationTime - GetTime() end    
    aura.unitCaster = "player"
    aura.isStealable = false
    aura.shouldConsolidate = false
    aura.spellId = 1
    aura.isTest = true
    tinsert(dummyAuras,aura)

    aura = {}
    aura.name = "Magic (Stealable)"
    aura.rank = 1
    aura.icon = "Interface\\Icons\\Temp.blp"
    aura.count = 1
    aura.dispelType = "Magic"
    aura.duration = 25
    aura.expirationTime = GetTime() + 25
    aura.remaining = function() return aura.expirationTime - GetTime() end    
    aura.unitCaster = "player"
    aura.isStealable = true
    aura.shouldConsolidate = false
    aura.spellId = 1
    aura.isTest = true
    tinsert(dummyAuras,aura)
    
    aura = {}
    aura.name = "Poison"
    aura.rank = 1
    aura.icon = "Interface\\Icons\\Temp.blp"
    aura.count = 1
    aura.dispelType = "Poison"
    aura.duration = 20
    aura.expirationTime = GetTime() + 20
    aura.remaining = function() return aura.expirationTime - GetTime() end
    aura.unitCaster = "player"
    aura.isStealable = false
    aura.shouldConsolidate = false
    aura.spellId = 1
    aura.isTest = true
    tinsert(dummyAuras,aura)

    return dummyAuras
end
