local WYM = _G.WymBuffs
local L = WYM.L
local LSM = LibStub("LibSharedMedia-3.0")

local anchorPoints = {
    ["TOP"]="TOP", ["RIGHT"]="RIGHT", ["BOTTOM"]="BOTTOM", ["LEFT"]="LEFT",
    ["TOPRIGHT"]="TOPRIGHT", ["TOPLEFT"]="TOPLEFT", ["BOTTOMLEFT"]="BOTTOMLEFT", ["BOTTOMRIGHT"]="BOTTOMRIGHT",
    ["CENTER"]="CENTER",
}

local tooltipAnchorPoints = {
    ["ANCHOR_BOTTOMLEFT"] = "BOTTOMLEFT", ["ANCHOR_CURSOR"] = "CURSOR", ["ANCHOR_LEFT"] = "LEFT",
    ["ANCHOR_NONE"] = "NONE", ["ANCHOR_RIGHT"] = "RIGHT", ["ANCHOR_TOPLEFT"] = "TOPLEFT",
    ["ANCHOR_TOPRIGHT"] = "TOPRIGHT",
}

local warningSounds = LSM:HashTable("sound")
local barTextures = LSM:HashTable("statusbar")
local fonts = LSM:HashTable("font")

local currentBuffFrameSettings
local currentBuffSettings

local defaultBuffColors = {
    ["none"] = { ["r"] = 1.0, ["g"] = 0.7, ["b"] = 0},
    ["Poison"] = { ["r"] = 0, ["g"] = 1, ["b"] = 0},
    ["Magic"] = { ["r"] = 0, ["g"] = 0, ["b"] = 1},
    ["Disease"] = { ["r"] = 0.55, ["g"] = 0.15, ["b"] = 0},
    ["Curse"] = { ["r"] = 1, ["g"] = 0, ["b"] = 1},
    
    ["warning"] = { ["r"] = 1, ["g"] = 0, ["b"] = 0 }, -- Used for putting a different coloured border round missing buffs
}

local function validateNumber(info, value)
    if not tonumber(value) then
        return L["The value for this settings has to be a number."]
    end
    return true
end

local buffFrameDefaultSettings = {

    enabled = true,
    unit = "player",
    type = "HELPFUL",
    displayType = "BARS",
    sortOrder = "ADDED",
    reverseSortOrder = false,
    growthDirection = "VERTICAL",
    rowsColumns = 1,
    
    positionAnchorPoint = "CENTER",
    positionRelativePoint = "CENTER",
    positionXOffset = 0,
    positionYOffset = 0,    

    hideOutOfCombat = false,
    showForNPCsOnly = false,
    
    playWarningSoundOnNewBuff = false,
    warningSoundOnNewBuff = "None",
    showWarningFlashOnNewBuff = false,
    warningFlashOnNewBuffSize = 150,
    
    showTestBuffs = false,
            
    barWidth = 200,
    barHeight = 20,
    barSpacing = 0,    
    barTexture = 'Blizzard',
    barBackgroundColorR = 0.5,
    barBackgroundColorG = 0.5,
    barBackgroundColorB = 0.5,
    barBackgroundColorA = 1.0,
    barLabelFont = "Friz Quadrata TT",
    barLabelFontSize = 12,
    barLabelFontOutline = "NONE",
    barLabelTextColorR = 1.0,
    barLabelTextColorG = 1.0,
    barLabelTextColorB = 1.0,
    barLabelTextColorA = 1.0,
    barLabelAlignment = "CENTER",        
    barShowIcon = true,
    barShowTime = true,
    barShowSpark = true,    
    
    iconLabelFont = "Friz Quadrata TT",
    iconLabelFontSize = 12,
    iconLabelFontOutline = "NONE",
    iconLabelTextColorR = 1.0,
    iconLabelTextColorG = 1.0,
    iconLabelTextColorB = 1.0,
    iconLabelTextColorA = 1.0,        
    iconLabelPosition = "HIDE",
    iconTimerFont = "Friz Quadrata TT",
    iconTimerFontSize = 12,
    iconTimerFontOutline = "NONE",
    iconTimerTextColorR = 1.0,
    iconTimerTextColorG = 1.0,
    iconTimerTextColorB = 1.0,
    iconTimerTextColorA = 1.0,        
    iconTimerPosition = "BELOW",
    iconTimerFormat = "DEFAULT",
    iconScale = 1.0,
    iconSpacing = 10,
    showIconBorder = true,
    iconBorderSize = 67,

    blacklist = false,
    showMissingBuffs = false,
    showOwnBuffsOnly = false,
    showDispellableOnly = false,
    showStealableOnly = false,
    showNoDurationBuffs = true,
    showDurationBuffs = true,
    showMaxDuration = 0,

    enableDebuffCategoryFilter = false,
    showPhysicalDamageTaken = false,
    showSpellDamageTaken = false,
    showBleedDamageTaken = false,
    showReducedAttackSpeed = false,
    showReducedCastingSpeed = false,
    showReducedArmor = false,
    showHealingDebuff = false,
    showPhysicalDamageReduction = false,

    showTooltip = true,
    tooltipAnchorPoint = "ANCHOR_CURSOR",
    tooltipScale = 0.9,    
}

local buffDefaultSettings = {
    useDefaultBuffColor = true,
    buffColorR = 0.0,
    buffColorG = 0.0,
    buffColorB = 0.0,
    buffColorA = 1.0,
    
    warnWhenExpiring = false,
    expirationWarningTime = 0,
    playWarningSound = false,
    warningSound = "None",
    warningBlinkBuff = false,
    warningBlinkBuffSpeed = 1.0,    
    onlyWarnInCombat = false,
    
    showAsMissingOnlyIfSpellExists = false,    
}

local options = {}

options.general = {

    handler = WYM,
    
    name = "WymBuffs",
    type = "group",
    desc = L["Settings"],
        
    args = {
        header = {            
            type = "description",
            order = 0,
            name = L["WymBuffs is an addon for keeping track of buffs on yourself or other units. Buffs are placed into frames. Each frame " ..
                     "tracks one unit for buffs or debuffs and can be configured with its own display and buff filter settings."],
        },
        
        enable = {            
            type = "toggle",
            order = 1,
            name = L["Enable"],
            width = "double",
            desc = L["Turns WymBuffs on or off."],
            get = function() return WYM:IsEnabled() end,
            set = function()
                    if WYM:IsEnabled() then
                        WYM:Disable()
                    else
                        WYM:Enable()
                    end
                  end,
        },
        
        selectBuffFrame = {
            type = "select",
            order = 3,
            name = L["Select Buff Frame"],
            desc = L["Select a buff frame to configure."],
            get = function()
                      if currentBuffFrameSettings then
                          return currentBuffFrameSettings.id
                      end
                  end,
            set = function(info, id)                      
                      currentBuffFrameSettings = WYM:GetBuffFrameSettings(id)
                  end,
            values = function()
                         local values = {}
                         for id, settings in pairs(WYM.db.profile.buffFrameSettings) do
                             values[id] = settings.name
                         end
                         return values
                     end,                  
        },
        
        newBuffFrame = {
            type = "input",
            order = 5,
            name = L["New Buff Frame"],
            desc = L["Enter the name of the frame you want to create here."],
            validate = function(info, name)
                           if name == "" then
                               return L["The frame name cannot be empty."]
                           end
                           for id, settings in pairs(WYM.db.profile.buffFrameSettings) do
                               if strlower(settings.name) == strlower(name) then
                                   return L["A frame with that name is already defined."]
                               end
                           end
                           return true
                       end,
            get = function() return "" end,
            set = "NewBuffFrame",                    
        },
        
        copySettings = {
            type = "select",
            order = 7,
            name = L["Copy Settings From"],
            desc = L["Copy the settings from another buff frame to the selected one."],
            get = nil,
            set = function(info, id)
                      local fromBuffFrameSettings = WYM:GetBuffFrameSettings(id)
                      for key, setting in pairs(fromBuffFrameSettings) do
                          if not (key == 'name') and not(key == 'id') and not (key == 'buffWatchList') then
                            currentBuffFrameSettings[key] = fromBuffFrameSettings[key]
                          end
                      end
                  end,
            values = function()
                         local values = {}
                         for id, settings in pairs(WYM.db.profile.buffFrameSettings) do
                             values[id] = settings.name
                         end
                         return values
                     end,
            disabled = function() return not currentBuffFrameSettings end,
        },
        
        filler = {
            type = "description",
            order = 8,
            name = "",
        },
        
        deleteBuffFrame = {
            type = "execute",
            order = 9,
            name = L["Delete Buff Frame"],
            desc = L["Delete the currently selected buff frame."],
            func = function()
                       WYM:DeleteBuffFrame(currentBuffFrameSettings.id)
                       WYM.db.profile.buffFrameSettings[currentBuffFrameSettings.id] = nil
                       currentBuffFrameSettings = nil
                       currentBuffSettings = nil
                       --WYM:UpdateBuffFrames()
                   end,
            disabled = function() return not currentBuffFrameSettings end,
        },

        hideBlizzardFrames = {
            type = "toggle",
            order = 10,
            width = "full",
            name = L["Hide Blizzard Frames"],
            desc = L["Hide the default buff and debuff frames."],
            get = function() return WYM.db.profile.hideBlizzardFrames end,
            set = function(info, value)
                      WYM.db.profile.hideBlizzardFrames = value
                      WYM:ToggleBlizzardFrames(value)
                  end,

        },
        
        generalSettings = {
            type = "group",
            name = L["General Settings"],
            inline = true,
            order = 11,
            
            args = {
                enabled = {
                    order = 0,
                    type = "toggle",
                    width = "full",
                    name = L["Enable"],
                    desc = L["Enable or disable this buff frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
            
                unit = {
                    order = 2,
                    type = "select",
                    name = L["Unit"],
                    desc = L["Sets the unit this frame should monitor for buffs/debuffs."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = { ["player"] = L["Player"], ["target"] = L["Target"], ["focus"] = L["Focus"], ["pet"] = L["Pet"], ["vehicle"] = L["Vehicle"], ["mouseover"] = L["Mouseover"], ["weapon"] = L["Weapon Buffs"] },
                    disabled = function() return not currentBuffFrameSettings end,
                },

                type = {
                    order = 4,
                    type = "select",
                    width = "half",
                    name = L["Type"],
                    desc = L["Sets the type of aura this frame should monitor the selected unit for."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = { ["HELPFUL"] = L["Buffs"], ["HARMFUL"] = L["Debuffs"] },
                    disabled = function() return (not currentBuffFrameSettings) or currentBuffFrameSettings.unit == "weapon" end,
                },
                                
                displayType = {
                    order = 6,
                    type = "select",
                    width = "half",
                    name = L["Display Type"],
                    desc = L["Display the buffs in this frame as bars or icons."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = { ["BARS"] = L["Bars"], ["ICONS"] = L["Icons"] },
                    disabled = function() return not currentBuffFrameSettings end,
                },            
                
                sortOrder = {
                    type = "select",
                    order = 8,
                    width = "half",
                    name = L["Buff Sort Order"],
                    desc = L["Sets the order in which the bars in this frame are sorted."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = { ["ADDED"] = L["Added"], ["DURATION"] = L["Duration"], ["NAME"] = L["Name"], ["TYPE"] = L["Type"] },
                    disabled = function() return not currentBuffFrameSettings end,
                },

                reverseSortOrder = {
                    type = "toggle",
                    order = 10,
                    name = L["Reverse Sort Order"],
                    desc = L["Add bars to this frame in the reverse of the selected sort order."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                growthDirection = {
                    type = "select",
                    order = 12,
                    name = L["Direction"],
                    desc = L["Sets the direction new buffs are added to this frame. The actual direction will depend on where the frame anchor point is."],                    
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = { ["VERTICAL"]="Vertical", ["HORIZONTAL"]="Horizontal", },
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                rowsColumns = {
                    type = "range",
                    order = 14,
                    name = L["Rows Or Columns"],
                    desc = L["Sets the number of rows (if direction is set to vertical) or columns (if direction is set to horizontal) in this frame."],
                    min = 1,
                    max = 40,
                    step = 1,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                hideOutOfCombat = {
                    type = "toggle",
                    order = 16,
                    width = "full",
                    name = L["Hide Out of Combat"],
                    desc = L["Only add buffs to this frame if the player is in combat."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                showForNPCsOnly = {
                    type = "toggle",
                    order = 18,
                    width = "full",
                    name = L["Only Watch NPCs"],
                    desc = L["Only add buffs to this frame if the unit monitored by this frame is an NPC."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },

                playWarningSoundOnNewBuff = {
                    type = "toggle",
                    order = 20,
                    name = L["Play Warning Sound"],
                    desc = L["Play a warning sound when a buff is added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
        
                warningSoundOnNewBuff = {
                    type = "select",
                    dialogControl = 'LSM30_Sound',
                    order = 22,
                    name = L["Warning Sound"],
                    desc = L["Select the sound to be played when a buff is added to this frame."],
                    values = warningSounds,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not currentBuffFrameSettings.playWarningSoundOnNewBuff end,
                },

                showWarningFlashOnNewBuff = {
                    type = "toggle",
                    order = 24,
                    name = L["Flash On New Buff"],
                    desc = L["Show a flash animation when a buff is added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },

                warningFlashOnNewBuffSize = {                    
                    type = "range",
                    order = 26,
                    name = L["Flash Size"],
                    desc = L["Sets the size of the flash animations on new buffs for this frame."],
                    min = 1,
                    max = 200,
                    step = 1,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not currentBuffFrameSettings.showWarningFlashOnNewBuff end,
                },

                showTestBuffs = {
                    type = "toggle",
                    order = 28,
                    name = L["Show Test Buffs"],
                    desc = L["Add a few test buff to help with configuration to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },                
            },
        },
        
        positionSettings = {
            type = "group",
            name = L["Position"],
            inline = true,
            order = 20,
            
            args = {
                positionAnchorPoint = {
                    order = 2,
                    type = "select",
                    name = L["Anchor Point"],
                    desc = L["Sets the anchor point of this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = anchorPoints,
                    disabled = function() return not currentBuffFrameSettings end,
                },
            
                positionRelativePoint = {
                    order = 4,
                    type = "select",
                    name = L["Relative Point"],
                    desc = L["Sets the relative point of the screen this frame is anchored to."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = anchorPoints,
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                positionXOffset = {
                    order = 6,
                    type = "input",
                    width = "half",
                    name = L["X offset"],
                    desc = L["X offset for the anchor point."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                positionYOffset = {
                    order = 8,
                    type = "input",
                    width = "half",
                    name = L["Y offset"],
                    desc = L["Y offset for the anchor point."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings end,
                },
            },
        },
        
        barSettings = {
            type = "group",
            name = L["Bar Layout"],
            inline = true,
            order = 30,
            
            args = {
                barWidth = {
                    type = "input",
                    width = "half",
                    order = 2,
                    name = L["Bar Width"],
                    desc = L["Sets the width of the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                },
                
                barHeight = {
                    type = "input",
                    width = "half",
                    order = 4,
                    name = L["Bar Height"],
                    desc = L["Sets the height of the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                },                

                barSpacing = {
                    type = "input",
                    width = "half",
                    order = 6,
                    name = L["Bar Spacing"],
                    desc = L["Sets the spacing between bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                },                

                barTexture = {
                    type = "select",
                    order = 8,
                    dialogControl = 'LSM30_Statusbar',
                    name = L["Bar Texture"],
                    desc = L["Sets the texture for the bars in this frame."],                    
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                    values = barTextures,
                },

                barBackgroundColor = {
                    type = "color",
                    order = 10,
                    name = L["Bar Background"],
                    desc = L["Sets the background color for the bars in this frame."],
                    hasAlpha = true,
                    get = "GetBuffFrameColorSetting",
                    set = "SetBuffFrameColorSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                },

                barLabelFont = {
                    type = "select",
                    dialogControl = 'LSM30_Font',
                    order = 12,
                    name = L["Bar Label Font"],
                    desc = L["Sets the font for the text on the bars in this frame."],
                    values = fonts,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                    
                },
                
                barLabelFontSize = {
                    type = "input",
                    width = "half",
                    order = 14,
                    name = L["Font Size"],
                    desc = L["Sets the font size for the text on the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                    
                },
    
                barLabelFontOutline = {
                    type = "select",
                    width = "half",
                    order = 16,
                    name = L["Outline"],
                    desc = L["Sets the outline of the font of the text on the bars in this frame."],
                    values = { ["NONE"] = L["None"], ["OUTLINE"] = L["Thin"], ["THICKOUTLINE"] = L["Thick"] },
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                    
                },
                
                barLabelTextColor = {
                    type = "color",
                    order = 18,
                    name = L["Bar Label Text Color"],
                    desc = L["Sets the color of the text on the bars in this frame."],
                    hasAlpha = true,
                    get = "GetBuffFrameColorSetting",
                    set = "SetBuffFrameColorSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,
                },
    
                barLabelAlignment = {
                    type = "select",
                    width = "half",
                    order = 20,
                    name = L["Alignment"],
                    desc = L["Sets the alignment of the text on the bars in this frame."],
                    values = { ["LEFT"] = L["Left"], ["CENTER"] = L["Center"], ["RIGHT"] = L["Right"] },
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                    
                },
                
                barShowIcon = {
                    type = "toggle",
                    order = 22,
                    width = "full",
                    name = L["Show Icon"],
                    desc = L["Show the buff icon next to the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                                        
                },                
               
                barShowTime = {
                    type = "toggle",
                    order = 24,
                    width = "full",
                    name = L["Show Time"],
                    desc = L["Show remaining time on the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                                        
                },                                                

                barShowSpark = {
                    type = "toggle",
                    order = 26,
                    width = "full",
                    name = L["Show Spark"],
                    desc = L["Show the position spark on the bars in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "BARS") end,                                        
                },
            },
        },

        iconSettings = {
            type = "group",
            name = L["Icon Layout"],
            inline = true,
            order = 40,
            
            args = {
                iconLabelFont = {
                    type = "select",
                    dialogControl = 'LSM30_Font',
                    order = 2,
                    name = L["Icon Label Font"],
                    desc = L["Sets the font for the buff name label for the icons in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                    values = fonts,
                },
                
                iconLabelFontSize = {
                    type = "input",
                    width = "half",
                    order = 4,
                    name = L["Font Size"],
                    desc = L["Sets the font size for the buff name label for the icons in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },
                
                iconLabelFontOutline = {
                    type = "select",
                    width = "half",
                    order = 6,
                    name = L["Outline"],
                    desc = L["Sets the outline for the buff name label for the icons in this frame."],
                    values = { ["NONE"] = L["None"], ["OUTLINE"] = L["Thin"], ["THICKOUTLINE"] = L["Thick"] },
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },

                iconLabelTextColor = {
                    type = "color",
                    order = 8,
                    name = L["Icon Label Text Color"],
                    desc = L["Sets the color of the text on the name label for the icons in this frame."],
                    hasAlpha = true,
                    get = "GetBuffFrameColorSetting",
                    set = "SetBuffFrameColorSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },
                
                iconLabelPosition = {
                    type = "select",
                    width = "half",
                    order = 10,
                    name = L["Label Position"],
                    desc = L["Sets the position of or hides the buff name label for the buffs in this frame."],                    
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                    values = { ["ABOVE"]="Above", ["BELOW"]="Below", ["LEFT"]="Left", ["RIGHT"]="Right", ["HIDE"]="Hide" },
                },            
                                            
                iconTimerFont = {
                    type = "select",
                    dialogControl = 'LSM30_Font',
                    order = 12,
                    name = L["Timer Font"],
                    desc = L["Sets the font for the timer for the icons in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                    values = fonts,
                },
                
                iconTimerFontSize = {
                    type = "input",
                    width = "half",
                    order = 14,
                    name = L["Font Size"],
                    desc = L["Sets the font size for the timer for the icons in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },

                iconTimerFontOutline = {
                    type = "select",
                    width = "half",
                    order = 16,
                    name = L["Outline"],
                    desc = L["Sets the outline for the timer for the icons in this frame."],
                    values = { ["NONE"] = L["None"], ["OUTLINE"] = L["Thin"], ["THICKOUTLINE"] = L["Thick"] },
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },

                iconTimerTextColor = {
                    type = "color",
                    order = 18,
                    name = L["Icon Timer Text Color"],
                    desc = L["Sets the color of the timer for the icons in this frame."],
                    hasAlpha = true,
                    get = "GetBuffFrameColorSetting",
                    set = "SetBuffFrameColorSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,                    
                },
                
                iconTimerPosition = {
                    type = "select",
                    width = "half",
                    order = 20,
                    name = L["Timer Position"],
                    desc = L["Sets the position of or hides the timer for the buffs in this frame."],                    
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                    values = { ["ABOVE"]="Above", ["BELOW"]="Below", ["LEFT"]="Left", ["RIGHT"]="Right", ["HIDE"]="Hide" },
                },

                iconTimerFormat = {
                    type = "select",
                    width = "half",
                    order = 22,
                    name = L["Timer Format"],
                    desc = L["Sets the format of the timer for the buffs in this frame."],                    
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                    values = { ["DEFAULT"]="Default", ["MM:SS"]="MM:SS" },
                },
                                
                iconScale = {                    
                    type = "range",
                    order = 24,
                    name = L["Icon Scale"],
                    desc = L["Sets the scale of the icons for this frame."],
                    min = 0.1,
                    max = 5.0,
                    step = 0.1,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                },
                                
                iconSpacing = {
                    type = "input",
                    width = "half",
                    order = 26,
                    name = L["Icon Spacing"],
                    desc = L["Sets the spacing between the icons in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                },
                
                showIconBorder = {
                    type = "toggle",
                    order = 28,
                    name = L["Show Border"],
                    desc = L["Show a colored border for buffs in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                },                
                
                iconBorderSize = {
                    order = 30,
                    type = "range",
                    name = L["Border Size"],
                    desc = L["Adjusts the size of the border texture to make it fit the icon better if a skin that adjusts the icon size is used."],
                    min = 1,
                    max = 200,
                    step = 1.0,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or not (currentBuffFrameSettings.displayType == "ICONS") end,
                },                
            },
        },

        buffWatchListSettings = {
            type = "group",
            name = L["Buff Watchlist And Settings"],
            inline = true,
            order = 50,

            args = {

                selectWatchedBuff = {
                    type = "select",
                    order = 2,
                    name = L["Buff Watchlist"],
                    desc = L["List of buffs that will be monitored by this frame. If this frame is watching temporary weapon buffs then these settings will not filter buffs, they " ..
                             "will only set color and warnings for the listed buffs."],
                    get = function()
                              if currentBuffSettings then
                                  return currentBuffSettings.name
                              end
                          end,
                    set = function(info, name)                      
                              currentBuffSettings = currentBuffFrameSettings.buffWatchList[name]
                          end,
                    values = function()
                                 local values = {}
                                 if currentBuffFrameSettings then
                                    for name, _ in pairs(currentBuffFrameSettings.buffWatchList) do
                                        values[name] = name
                                    end
                                 end
                                 return values
                             end,
                    disabled = function() return not currentBuffFrameSettings end,
                },

                addBuffToWatchList = {
                    type = "input",
                    order = 4,
                    name = L["Add Buff To Watchlist"],
                    desc = L["Enter the name of the buff you want this frame to monitor."],
                    validate = function(info, newname)
                                   for name, _ in pairs(currentBuffFrameSettings.buffWatchList) do
                                       if strlower(name) == strlower(newname) then
                                           return L["A buff with that name is already being monitored by this frame."]
                                       end
                                   end
                                   return true
                               end,
                    get = function() return "" end,
                    set = "NewBuffSettings",
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                deleteBuffFromWatchList = {
                    type = "execute",
                    order = 6,
                    name = L["Delete Buff"],
                    desc = L["Delete the currently selected buff from the list of buffs monitored by this frame."],
                    func = function()
                               currentBuffFrameSettings.buffWatchList[currentBuffSettings.name] = nil
                               currentBuffSettings = nil
                               WYM:UpdateBuffFrames()
                           end,
                    disabled = function() return not currentBuffSettings end,
                },

                filler = {
                    type = "description",
                    order = 7,
                    name = "",
                },

                useDefaultBuffColor = {
                    type = "toggle",
                    order = 8,
                    name = L["Use Default Buff Color"],
                    desc = L["Use the default color for the debufftype of the buff for bars or icons for this buff in this frame."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings end,
                },                

                buffColor = {
                    type = "color",
                    order = 10,
                    name = L["Buff Color"],
                    desc = L["Sets the color for the bar or icon for this buff in this frame."],
                    hasAlpha = true,
                    get = "GetBuffColorSetting",
                    set = "SetBuffColorSetting",
                    disabled = function() return not currentBuffSettings
                                                 or currentBuffSettings.useDefaultBuffColor
                                          end,
                },
                
                warnWhenExpiring = {
                    type = "toggle",
                    order = 12,
                    name = L["Warn When Expiring"],
                    desc = L["Warn when this buff is close to expiring."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings end,
                },                
                              
                expirationWarningTime = {
                    type = "input",
                    width = "half",
                    order = 14,
                    name = L["Warning Time"],
                    desc = L["Sets how many seconds before the buff expires the warning should be performed."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffSettings or not currentBuffSettings.warnWhenExpiring end,                    
                },
                
                playWarningSound = {
                    type = "toggle",
                    order = 16,
                    name = L["Play Warning Sound"],
                    desc = L["Play a warning sound when this buff is close to expiring."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings or not currentBuffSettings.warnWhenExpiring end,
                },                

                warningSound = {
                    type = "select",
                    dialogControl = 'LSM30_Sound',
                    order = 18,
                    name = L["Warning Sound"],
                    desc = L["Select the sound to be played when this buff is close to expiring."],
                    values = warningSounds,
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings or not currentBuffSettings.playWarningSound end,
                },

                warningBlinkBuff = {
                    type = "toggle",
                    order = 20,
                    name = L["Blink Buff"],
                    desc = L["Blink this buff when it is close to expiring."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings or not currentBuffSettings.warnWhenExpiring end,
                },                

                warningBlinkBuffSpeed = {
                    order = 22,
                    type = "range",
                    name = L["Blink speed"],
                    desc = L["Sets the speed this buff will blink at when it is close to expiring."],
                    min = 0.01,
                    max = 5.0,
                    step = 0.01,
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings or not currentBuffSettings.warningBlinkBuff end,
                },

                onlyWarnInCombat = {
                    type = "toggle",
                    order = 24,
                    width = "full",
                    name = L["Only Warn In Combat"],
                    desc = L["Only perform a warning when this buff is close to expiring if you are in combat."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings end,
                },                

                showAsMissingOnlyIfSpellExists = {
                    type = "toggle",
                    order = 26,
                    width = "full",
                    name = L["Show Missing Only If Spell Exists"],
                    desc = L["Only show this buff as missing if your spellbook contains a spell with the same name as the buff."],
                    get = "GetBuffSetting",
                    set = "SetBuffSetting",
                    disabled = function() return not currentBuffSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                
            },
        },

        buffFilterSettings = {
            type = "group",
            name = L["Buff Filter"],
            inline = true,
            order = 60,

            args = {

                blacklist = {
                    type = "toggle",
                    order = 2,
                    width = "full",
                    name = L["Blacklist"],
                    desc = L["If this option is selected, all buffs except those in the buff watch list will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showMissingBuffs = {
                    type = "toggle",
                    order = 4,
                    width = "full",
                    name = L["Show Missing Buffs"],
                    desc = L["If this option is selected, only the buffs in the buff watch list that are missing will be added to this frame. " ..
                             "If this frame is watching temporary weapon buffs, this setting will instead mean that weapons missing temporary buffs " ..
                             "will be included in this frame along with weapons that have temporary buffs."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },                

                showOwnBuffsOnly = {
                    type = "toggle",
                    order = 6,
                    width = "full",
                    name = L["Show Own Buffs Only"],
                    desc = L["If this option is selected, only buffs provided by you will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showDispellableOnly = {
                    type = "toggle",
                    order = 8,
                    width = "full",
                    name = L["Show Dispellable Buffs Only"],
                    desc = L["If this option is selected, only dispellable buffs will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showStealableOnly = {
                    type = "toggle",
                    order = 10,
                    width = "full",
                    name = L["Show Stealable Buffs Only"],
                    desc = L["If this option is selected, only buffs that can be stolen will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showNoDurationBuffs = {
                    type = "toggle",
                    order = 12,
                    width = "full",
                    name = L["Show Buffs With No Duration"],
                    desc = L["If this option is selected, buffs that have no duration will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showDurationBuffs = {
                    type = "toggle",
                    order = 14,
                    width = "full",
                    name = L["Show Buffs With Duration"],
                    desc = L["If this option is selected, buffs that have a duration will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                

                showMaxDuration = {
                    type = "input",
                    order = 16,
                    width = "half",
                    name = L["Max Duration"],
                    desc = L["If this setting is greater than zero, only buffs with a duration shorter than or equal to this number of seconds will be added to this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    validate = validateNumber,                    
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },                
            },
        },
        
        debuffCategorySettings = {
            type = "group",
            name = L["Debuff Categories"],
            inline = true,
            order = 70,
            
            args = {
                enableDebuffCategoryFilter = {
                    type = "toggle",
                    order = 0,
                    width = "double",
                    name = L["Enable"],
                    desc = L["Enables the debuff category filter. If this setting is enabled, the filters in the buff filter section and the buff watch list will be ignored."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" end,
                },

                showPhysicalDamageTaken = {
                    type = "toggle",
                    order = 2,
                    width = "double",
                    name = L["Physical Damage Taken"],
                    desc = L["Blood Frenzy, Savage Combat, Brittle Bones, Ravage or Acid Spit."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showSpellDamageTaken = {
                    type = "toggle",
                    order = 4,
                    width = "double",
                    name = L["Spell Damage Taken"],
                    desc = L["Master Poisoner, Curse of the Elements, Ebon Plaguebringer, Earth and Moon, Fire Breath or Lightning Breath."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showBleedDamageTaken = {
                    type = "toggle",
                    order = 6,
                    width = "double",
                    name = L["Bleed Damage Taken"],
                    desc = L["Mangle, Hemorrhage, Blood Frenzy, Tendon Rip, Gore or Stampede."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showReducedAttackSpeed = {
                    type = "toggle",
                    order = 8,
                    width = "double",
                    name = L["Reduced Attack Speed"],
                    desc = L["Tunder Clap, Icy Touch, Infected Wounds, Judgements of the Just, Riposte, Earth Shock, Tailspin or Dust Cloud."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showReducedCastingSpeed = {
                    type = "toggle",
                    order = 10,
                    width = "double",
                    name = L["Reduced Casting Speed"],
                    desc = L["Curse of Tongues, Mind-Numbing Poison, Slow, Necrotic Strike, Spore Cloud or Lava Breath."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showReducedArmor = {
                    type = "toggle",
                    order = 12,
                    width = "double",
                    name = L["Reduced Armor"],
                    desc = L["Sunder Armor, Expose Armor, Faerie Fire (Feral), Corrosive Spit or Tear Armor."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },

                showHealingDebuff = {
                    type = "toggle",
                    order = 14,
                    width = "double",
                    name = L["Healing Received"],
                    desc = L["Mortal Strike, Furious Attacks, Improved Mind Blast, Wound Poison, Widow Venom, Permafrost, Legion Strike or Monstrous Bite."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },
                
                showPhysicalDamageReduction = {
                    type = "toggle",
                    order = 16,
                    width = "double",
                    name = L["Physical Damage Reduction"],
                    desc = L["Demoralizing Shout, Demoralizing Roar (Druid), Vindication, Scarlet Fever, Curse of Weakness or Demoralizing Roar (Hunter pet)."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings or currentBuffFrameSettings.unit == "weapon" or not currentBuffFrameSettings.enableDebuffCategoryFilter end,
                },
            },
        },

        tooltipSettings = {
            type = "group",
            name = L["Tooltip"],
            inline = true,
            order = 90,
            
            args = {
                showTooltip = {
                    type = "toggle",
                    order = 0,
                    width = "double",
                    name = L["Enable"],
                    desc = L["Show a tooltip with information about the buff when you mouseover the buffs in this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                tooltipAnchorPoint = {
                    order = 2,
                    type = "select",
                    name = L["Anchor Point"],
                    desc = L["Sets the anchor point of the tooltip for this frame."],
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    values = tooltipAnchorPoints,
                    disabled = function() return not currentBuffFrameSettings end,
                },
                
                tooltipScale = {
                    order = 4,
                    type = "range",
                    name = L["Tooltip Scale"],
                    desc = L["Sets the scale of the tooltip for this frame."],
                    min = 0.1,
                    max = 2.0,
                    step = 0.1,
                    get = "GetBuffFrameSetting",
                    set = "SetBuffFrameSetting",
                    disabled = function() return not currentBuffFrameSettings end,
                },
            },
        },        
    },
}

function WYM:OnInitialize()

    WYM.db = LibStub("AceDB-3.0"):New("WymBuffsDB", defaults, "Default")
    
    self:InitProfile()
        
    LibStub("AceConfig-3.0"):RegisterOptionsTable("WymBuffs", options.general)
    
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("WymBuffs", options.general.name)
        
    options.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(WYM.db)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("WymBuffsProfiles", options.profiles)
    self.optionsFrame.profiles = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("WymBuffsProfiles", options.profiles.name, options.general.name)
    WYM.db.RegisterCallback(self, "OnProfileChanged","ProfileChanged")    
    WYM.db.RegisterCallback(self, "OnProfileCopied","ProfileChanged")
    
    self:RegisterChatCommand(L["wymbuffs"], "ChatCommand")
    self:RegisterChatCommand(L["wb"], "ChatCommand")
  
    self:InitBuffFrames()  

    self:ToggleBlizzardFrames(WYM.db.profile.hideBlizzardFrames)
    self:UpdateBuffFrames()
end

function WYM:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(WYM, L["wb"], "WymBuffs", input)
        LibStub("AceConfigCmd-3.0").HandleCommand(WYM, L["wymbuffs"], "WymBuffs", input)
    end
end

function WYM:ProfileChanged(db,profile)
    -- InitProfile is only really needed when we make a new profile
    self:InitProfile()
    self:InitBuffFrames()
    
    currentBuffFrameSettings = nil
    currentBuffSettings = nil
end

function WYM:InitProfile() 
    -- This will be a mapping of buff name <-> spell id so that the missing buffs option
    -- will be able to find the right texture for buffs that it can't see at the moment.
    -- The slight disadvantage of doing it this way is that the addon has to have seen
    -- the buff once to be able to show the icon.
    if not WYM.db.profile.missingSpellId then
        WYM.db.profile.missingSpellId = {}
    end

    -- This table will hold the settings for the buff frames we want to set up.
    if not WYM.db.profile.buffFrameSettings then
        WYM.db.profile.buffFrameSettings = {}    
    end
end

function WYM:InitBuffFrames()
    for id,frame in pairs(self:GetBuffFrames()) do
        self:DeleteBuffFrame(id)
    end
     
    for id, settings in pairs(WYM.db.profile.buffFrameSettings) do
        local frame = self:CreateBuffFrame(id)
        frame.settings = settings
    end
    
    self:UpdateBuffFrames()
end

function WYM:NewBuffFrame(info,name)
    local newFrameSettings = {}
    newFrameSettings.name = strtrim(name)
    newFrameSettings.id = "WymBuffs" .. newFrameSettings.name
  
    for key, value in pairs(buffFrameDefaultSettings) do
        newFrameSettings[key] = value
    end
    
    newFrameSettings.buffWatchList = {}
    
    WYM.db.profile.buffFrameSettings[newFrameSettings.id] = newFrameSettings
    currentBuffFrameSettings = WYM.db.profile.buffFrameSettings[newFrameSettings.id]
    
    local frame = self:CreateBuffFrame(newFrameSettings.id)
    frame.settings = newFrameSettings

    return newFrameSettings
end

function WYM:ToggleBlizzardFrames(hide)
    -- This won't actually update the list of buffs in the frame until there's a change but
    -- we can live with that
    if hide then
	BuffFrame:Hide()
	TemporaryEnchantFrame:Hide()
	ConsolidatedBuffs:Hide()
	BuffFrame:UnregisterAllEvents()
    else
	BuffFrame:Show()
	if GetCVarBool("consolidateBuffs") then
		ConsolidatedBuffs:Show()
	end
	TemporaryEnchantFrame:Show()
	BuffFrame:RegisterEvent("UNIT_AURA")
    end
end

function WYM:NewBuffSettings(info, name)
    local newBuffSettings = {}
    newBuffSettings.name = name

    for key, value in pairs(buffDefaultSettings) do
        newBuffSettings[key] = value
    end
    
    currentBuffFrameSettings.buffWatchList[newBuffSettings.name] = newBuffSettings
    currentBuffSettings = currentBuffFrameSettings.buffWatchList[newBuffSettings.name]
    
    self:UpdateBuffFrames()
    
    return newBuffSettings
end

function WYM:GetBuffFrameSettings(id)
    return WYM.db.profile.buffFrameSettings[id]
end 

function WYM:GetBuffFrameSetting(info)
    return self:GetSetting(info, currentBuffFrameSettings)
end

function WYM:SetBuffFrameSetting(info, value)
    self:SetSetting(info, value, currentBuffFrameSettings)
end

function WYM:GetBuffSetting(info)
    return self:GetSetting(info, currentBuffSettings)
end

function WYM:SetBuffSetting(info, value)
    self:SetSetting(info, value, currentBuffSettings)
end

function WYM:GetSetting(info,table)
    if table then
        local setting = info[#info] -- See AceConfig docs for why this works
        return table[setting]
    end
end

function WYM:SetSetting(info, value, table)
    if table then
        local setting = info[#info]
        table[setting] = value
        self:UpdateBuffFrames()
    end
end

function WYM:GetBuffFrameColorSetting(info)
    return self:GetColorSetting(info, currentBuffFrameSettings)
end

function WYM:SetBuffFrameColorSetting(info, r, g, b, alpha)
    self:SetColorSetting(info, r, g, b, alpha, currentBuffFrameSettings)
end

function WYM:GetBuffColorSetting(info)
    return self:GetColorSetting(info, currentBuffSettings)
end

function WYM:SetBuffColorSetting(info, r, g, b, alpha)
    self:SetColorSetting(info, r, g, b, alpha, currentBuffSettings)
end

function WYM:GetColorSetting(info, table)
    if table then
        local setting = info[#info]
        return table[setting .. "R"],
               table[setting .. "G"],
               table[setting .. "B"],
               table[setting .. "A"]
    end
end

function WYM:SetColorSetting(info, r, g, b, alpha, table)
    local setting = info[#info]
    table[setting .. "R"] = r
    table[setting .. "G"] = g
    table[setting .. "B"] = b
    table[setting .. "A"] = alpha
    self:UpdateBuffFrames()
end

function WYM:GetBuffColor(frame, buff)
    local buffSettings = frame.settings.buffWatchList[buff.name]
    
    if buffSettings and not buffSettings.useDefaultBuffColor then
        return buffSettings.buffColorR, buffSettings.buffColorG, buffSettings.buffColorB, buffSettings.buffColorA
    else
        local color = defaultBuffColors[buff.dispelType]
        if not color then
            color = defaultBuffColors["none"]
        end
        return color.r, color.g, color.b, 1
    end
end

function WYM:GetBarTexture(name)
    return barTextures[name]
end

function WYM:GetFont(name)
    return fonts[name]
end

function WYM:GetWarningSound(name)
    return warningSounds[name]
end
