local WYM = _G.WymBuffs
local L = WYM.L

local dummyTooltip
local slotEnchants = { ["MainHandSlot"] = {}, ["SecondaryHandSlot"] = {}, ["RangedSlot"] = {}, }

local function GetTemporaryWeaponEnchantInfo(slot)
    -- Not really sure if this is the proper way to do this, I found this on "The Internet (tm)"
    if not dummyTooltip then
        dummyTooltip = CreateFrame("GameTooltip")
        dummyTooltip:SetOwner(WorldFrame, "ANCHOR_NONE")        
        dummyTooltip.lines = {}
          
        for i = 1, 30 do
            local left = dummyTooltip:CreateFontString()
            left:SetFontObject(GameFontNormal)
            local right = dummyTooltip:CreateFontString()
            right:SetFontObject(GameFontNormal)        
            
            dummyTooltip:AddFontStrings(left, right)
            dummyTooltip.lines[i] = { left, right }
        end
    end
    
    dummyTooltip:ClearLines()    
    dummyTooltip:SetInventoryItem("player",GetInventorySlotInfo(slot))

    local type
    
    for i = 1, dummyTooltip:NumLines() do
        local lineType = dummyTooltip.lines[i][2]:GetText()
        if not type and lineType then
            type = lineType
        end
        
        local text = dummyTooltip.lines[i][1]:GetText()
  
        if text then
            local buff_name = text:match("^(.+) %(%d+ [^$)]+%)$")            
            if buff_name then
                return buff_name, type
            end
        end
    end
    
    if slot == "MainHandSlot" then
        return "Main Hand", type
    end
    if slot == "SecondaryHandSlot" then
        return "Off Hand", type
    end
    if slot == "RangedSlot" then
        return "Ranged", type
    end
end

local function GetTemporaryWeaponEnchant(slot,expiration, charges)
    local enchant = slotEnchants[slot]
    local name, weaponType = GetTemporaryWeaponEnchantInfo(slot)
    
    enchant.name = name
    enchant.weaponType = weaponType
    enchant.rank = 1
    enchant.icon = GetItemIcon(GetInventoryItemID("player",GetInventorySlotInfo(slot)))
    enchant.count = charges or 0    
    if expiration then
        enchant.dispelType = "none"
        enchant.duration = expiration / 1000
        enchant.expirationTime = enchant.duration + GetTime()
        enchant.remaining = function()
            return enchant.expirationTime - GetTime()
        end
    else
        enchant.dispelType = "warning"
        enchant.duration = 0
        enchant.expirationTime = 0
        enchant.remaining = function() return -1 end    
    end
    enchant.unitCaster = "player"
    enchant.isStealable = false
    enchant.shouldConsolidate = false
    enchant.spellId = -1
    enchant.inventoryItemSlot = slot
    enchant.isTemporaryWeaponEnchant = true
    return enchant
end

function WYM:GetTemporaryWeaponEnchants(frame)
    local enchants = frame.buffs
    local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges, hasRangedEnchant, rangedExpiration, rangedCharges = GetWeaponEnchantInfo()

    wipe(slotEnchants["MainHandSlot"])
    wipe(slotEnchants["SecondaryHandSlot"])
    wipe(slotEnchants["RangedSlot"])
    
    if hasMainHandEnchant then
        tinsert(enchants,GetTemporaryWeaponEnchant("MainHandSlot",mainHandExpiration, mainHandCharges))
    elseif frame.settings.showMissingBuffs and GetInventoryItemID("player",GetInventorySlotInfo("MainHandSlot")) then
        local enchant = GetTemporaryWeaponEnchant("MainHandSlot")
        tinsert(enchants,enchant)
    end
    
    if hasOffHandEnchant then
        tinsert(enchants,GetTemporaryWeaponEnchant("SecondaryHandSlot",offHandExpiration, offHandCharges))
    elseif frame.settings.showMissingBuffs and GetInventoryItemID("player",GetInventorySlotInfo("SecondaryHandSlot")) then
        local enchant = GetTemporaryWeaponEnchant("SecondaryHandSlot")
        if enchant.weaponType and not (enchant.weaponType == L["Shield"]) then
            tinsert(enchants,enchant)
        end
    end
    
    if hasRangedEnchant then
        tinsert(enchants,GetTemporaryWeaponEnchant("RangedSlot",rangedExpiration, rangedCharges))
    elseif frame.settings.showMissingBuffs and GetInventoryItemID("player",GetInventorySlotInfo("RangedSlot")) then
        local _, classFileName = UnitClass("player")
        if classFileName == "ROGUE" then
            -- As far as we know, only rogues can put temporary weapon buffs on ranged slot items, and only on thrown weapons
            local enchant = GetTemporaryWeaponEnchant("RangedSlot")
            if enchant.weaponType == L["Thrown"] then
                tinsert(enchants,enchant)
            end
        end
    end

    if hasMainHandEnchant or hasOffHandEnchant or hasRangedEnchant then    
        -- It seems to be tricky to track refreshes of temporary weapon enchants through events. Weapon enchants
        -- don't always seem to be updated when the UNIT_INVENTORY_CHANGED events are fired, or something. So until we
        -- figure out a better way, we'll just have a function that ticks for a second after the event to and checks the
        -- temporary weapon enchants to make sure they get updated.
        if not frame:GetScript("OnUpdate") then
            frame:SetScript("OnUpdate",function ()
                                            if not frame.updating then
                                                frame.updating = GetTime() + 1.000
                                            end
                                            if frame.updating >= GetTime() then
                                                WYM:GetTemporaryWeaponEnchants(frame)
                                                WYM:UpdateBuffFrames("weapon")
                                            else
                                                frame.updating = nil
                                                frame:SetScript("OnUpdate", nil)
                                            end
                                        end)
        end
    elseif frame:GetScript("OnUpdate") then
        frame:SetScript("OnUpdate", nil)
    end
    
    return enchants    
end

