local WYM = _G.WymBuffs

local TOOLTIP_FRAME = CreateFrame("GameTooltip", "WymBuffsToolTip", nil, "GameTooltipTemplate");

local function OnUpdateFunc()
    local buff = TOOLTIP_FRAME.buff
    -- We want to update the tooltip on the screen so that it shows the correct time remaining but
    -- don't bother doing that more often than once every second.
    local remaining = floor(buff.expirationTime - GetTime())
    if remaining < TOOLTIP_FRAME.remaining then
	TOOLTIP_FRAME:SetUnitAura(buff.unit, buff.index, buff.filter)
	TOOLTIP_FRAME.remaining = remaining
    end
end

function WYM:EnableTooltip(frame, buff)
    if frame.settings.showTooltip then
	buff.widget:EnableMouse(true)
	buff.widget:SetScript("OnEnter",function(self) WYM:ShowTooltip(frame, buff) end)
	buff.widget:SetScript("OnLeave",function(self) WYM:HideTooltip() end)
    else
	buff.widget:EnableMouse(false)
	buff.widget:SetScript("OnEnter",nil)
	buff.widget:SetScript("OnLeave",nil)	
    end
end

function WYM:ShowTooltip(frame, buff)
    -- Testbars, testicons aren't real buffs and shouldn't get tooltips. The same goes for missing buffs
    if not buff.isTest and not buff.isMissing then
	TOOLTIP_FRAME:SetOwner(buff.widget,frame.settings.tooltipAnchorPoint)
        if buff.isTemporaryWeaponEnchant then
            TOOLTIP_FRAME:SetInventoryItem("player",GetInventorySlotInfo(buff.inventoryItemSlot))
        else
            TOOLTIP_FRAME:SetUnitAura(buff.unit, buff.index, buff.filter)
            TOOLTIP_FRAME:SetScript("OnUpdate", OnUpdateFunc)
            TOOLTIP_FRAME.remaining = buff.duration
        end
	TOOLTIP_FRAME:SetScale(frame.settings.tooltipScale)	
	TOOLTIP_FRAME.buff = buff
        TOOLTIP_FRAME.frame = frame
	TOOLTIP_FRAME:Show()
    end
end

function WYM:HideTooltip()
    TOOLTIP_FRAME:SetScript("OnUpdate", nil)
    TOOLTIP_FRAME.buff = nil
    TOOLTIP_FRAME.frame = nil
    TOOLTIP_FRAME.remaining = nil
    TOOLTIP_FRAME:Hide()
end
